/*
 * Copyright (c) 2009-2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#include "WAppWindow.h"
#include "WDebugBase.h"
#include "WView.h"

#include "ContactsAppControl.h"
#include "CtDetailView.h"
#include "CtInputView.h"
#include "CtDetailApp.h"

CtDetailApp::CtDetailApp()
	: m_Naviframe(NULL), m_Request(NULL)
{
	contacts_connect();
	elm_app_base_scale_set(2.6);
}

CtDetailApp::~CtDetailApp()
{
	if (m_Request) {
		app_control_destroy(m_Request);
	}
	contacts_disconnect();
}

bool CtDetailApp::onCreate()
{
	ContactsCommon::loadLocalization();
	attachWindow( new WAppWindow("Contacts-Details", ELM_WIN_BASIC));

	m_Naviframe = new WNaviframe();
	m_Naviframe->setOnLastItemPop([](bool *popOut) {
		ui_app_exit();
	});
	getWindow()->attachBaseUiObject(m_Naviframe);

	return true;
}

void CtDetailApp::onAppControl(app_control_h request, bool firstLaunch)
{
	WENTER();

	WView *view = NULL;
	char *operation = NULL;

	app_control_clone(&m_Request, request);
	app_control_get_operation(m_Request, &operation);

	if (operation) {
		int personId = 0;

		char *personIdStr = NULL;
		app_control_get_extra_data(request, APP_CONTROL_DATA_ID, &personIdStr);
		if (personIdStr) {
			personId = atoi(personIdStr);
			free(personIdStr);
		}

		if (strcmp(operation, APP_CONTROL_OPERATION_VIEW) == 0) {
			if (personId > 0) {
				view = new CtDetailView(personId);
			}
		} else if (strcmp(operation, APP_CONTROL_OPERATION_ADD) == 0) {
			view = createInputView(personId, m_Request);
		} else if(strcmp(operation, APP_CONTROL_OPERATION_EDIT) == 0) {
			if (personId == 0) {
				using namespace std::placeholders;
				CtListView *listView = new CtListView(LIST_SELECT_SINGLE, LIST_RESULT_ID, 0);
				listView->setOnSingleResult(std::bind(&CtDetailApp::onContactPick, this, _1, _2));
				view = listView;
			} else {
				view = createInputView(personId, m_Request);
			}
		}

		free(operation);
	}

	if (view) {
		m_Naviframe->push(view);
	} else {
		app_control_h reply;
		app_control_create(&reply);
		app_control_reply_to_launch_request(reply, m_Request, APP_CONTROL_RESULT_FAILED);
		app_control_destroy(reply);
		ui_app_exit();
	}

	WLEAVE();
}

void CtDetailApp::onContactPick(const char *result, CtListResultType resultType)
{
	WENTER();

	int personId = result ? atoi(result) : 0;
	if (personId < 1) {
		ui_app_exit();
	}

	CtInputView *view = createInputView(personId, m_Request);
	m_Naviframe->push(view);
}

CtInputView * CtDetailApp::createInputView(int personId, app_control_h request)
{
	char *name = NULL;
	char *number = NULL;
	char *email = NULL;
	char *url = NULL;

	app_control_get_extra_data(request, APP_CONTROL_DATA_NAME, &name);
	app_control_get_extra_data(request, APP_CONTROL_DATA_PHONE, &number);
	app_control_get_extra_data(request, APP_CONTROL_DATA_EMAIL, &email);
	app_control_get_extra_data(request, APP_CONTROL_DATA_URL, &url);

	CtInputView *inputView = NULL;
	if (name) {
		inputView = new CtInputView(name, number, email, url);
	} else {
		inputView = new CtInputView(personId, number, email, url);
	}

	inputView->setOnResult([request](int personId, CtResultType result) {
		replyPersonId(personId, request, app_control_result_e(result));
	});

	free(name);
	free(number);
	free(email);
	free(url);

	return inputView;
}

void CtDetailApp::replyPersonId(int personId, app_control_h request, app_control_result_e result)
{
	app_control_h reply;
	app_control_create(&reply);

	if (result == APP_CONTROL_RESULT_SUCCEEDED) {
		char itemId[CT_TEXT_SHORT_LEN];
		snprintf(itemId, sizeof(itemId), "%d", personId);
		app_control_add_extra_data(reply, APP_CONTROL_DATA_ID, itemId);
	}

	app_control_reply_to_launch_request(reply, request, result);
	app_control_destroy(reply);
}
