/*
 * Copyright (c) 2009-2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#include <contacts.h>

#include "CtListApp.h"
#include "ContactsAppControl.h"

#include "WAppWindow.h"
#include "WDebugBase.h"

namespace
{
	const char *resultTypes[] = {
		APP_CONTROL_DATA_TYPE_ID,
		APP_CONTROL_DATA_TYPE_PHONE,
		APP_CONTROL_DATA_TYPE_PHONE_CALL,
		APP_CONTROL_DATA_TYPE_PHONE_MESSAGE,
		APP_CONTROL_DATA_TYPE_EMAIL,
		APP_CONTROL_DATA_TYPE_VCARD
	};
}

CtListApp::CtListApp()
	: m_Naviframe(NULL), m_Request(NULL),
	  m_SelectMode(LIST_SELECT_SINGLE), m_ResultType(LIST_RESULT_ID)
{
	contacts_connect();
}

CtListApp::~CtListApp()
{
	if (m_Request) {
		app_control_destroy(m_Request);
	}
	contacts_disconnect();
}

bool CtListApp::onCreate()
{
	ContactsCommon::loadLocalization();
	attachWindow(new WAppWindow("Contacts-List", ELM_WIN_BASIC));

	m_Naviframe = new WNaviframe();
	m_Naviframe->setOnLastItemPop([](bool *popOut) {
		ui_app_exit();
	});
	getWindow()->attachBaseUiObject(m_Naviframe);

	return true;
}

void CtListApp::onAppControl(app_control_h request, bool firstLaunch)
{
	WHIT();
	CtListView *view = NULL;
	char *operation = NULL;

	app_control_clone(&m_Request, request);
	app_control_get_operation(m_Request, &operation);

	app_control_launch_mode_e launchMode = APP_CONTROL_LAUNCH_MODE_SINGLE;
	app_control_get_launch_mode(m_Request, &launchMode);

	if (launchMode == APP_CONTROL_LAUNCH_MODE_SINGLE) {
		evas_object_smart_callback_add(getWindow()->getEvasObj(), "iconified",
				&CtListApp::onLowered, this);
	}

	if (operation) {
		if (strcmp(operation, APP_CONTROL_OPERATION_PICK) == 0) {
			m_SelectMode = getSelectMode(m_Request);
			m_ResultType = getResultType(m_Request);
			view = new CtListView(m_SelectMode, m_ResultType, getSelectLimit(m_Request));

			using namespace std::placeholders;
			if (m_SelectMode == LIST_SELECT_SINGLE) {
				view->setOnSingleResult(std::bind(&CtListApp::replySingleResult, this, _1, _2));
			} else {
				view->setOnMultiResult(std::bind(&CtListApp::replyMultiResult, this, _1, _2));
			}
		} else if (strcmp(operation, APP_CONTROL_OPERATION_UPDATE) == 0) {
			view = new CtListView(LIST_SELECT_SINGLE_OR_CREATE, LIST_RESULT_ID, 0);
			view->setOnSingleResult([this](const char *id, CtListResultType type) {
				if (id) {
					if (atoi(id) == 0) {
						app_control_set_operation(m_Request, APP_CONTROL_OPERATION_ADD);
					} else {
						app_control_set_operation(m_Request, APP_CONTROL_OPERATION_EDIT);
						app_control_add_extra_data(m_Request, APP_CONTROL_DATA_ID, id);
					}

					app_control_send_launch_request(m_Request, NULL, NULL);
				}
			});
		}

		free(operation);
	}

	if (view) {
		m_Naviframe->push(view);
	} else {
		replyFailure();
		ui_app_exit();
	}
}

void CtListApp::onLowered(void *data, Evas_Object *obj, void *event_info)
{
	CtListApp *app = (CtListApp *) data;
	app->replyFailure();
	ui_app_exit();
}

CtListSelectMode CtListApp::getSelectMode(app_control_h request)
{
	char *selectMode = NULL;
	app_control_get_extra_data(request, APP_CONTROL_DATA_SELECTION_MODE, &selectMode);

	if (selectMode) {
		if (strcmp(selectMode, APP_CONTROL_DATA_SELECTION_MODE_SINGLE) == 0) {
			return LIST_SELECT_SINGLE;
		} else if (strcmp(selectMode, APP_CONTROL_DATA_SELECTION_MODE_MULTIPLE) == 0) {
			return LIST_SELECT_MULTIPLE;
		}

		free(selectMode);
	}

	return LIST_SELECT_SINGLE;
}

CtListResultType CtListApp::getResultType(app_control_h request)
{
	char *resultType = NULL;
	app_control_get_extra_data(request, APP_CONTROL_DATA_TYPE, &resultType);

	if (resultType) {
		if (strcmp(resultType, APP_CONTROL_DATA_TYPE_ID) == 0) {
			return LIST_RESULT_ID;
		} else if (strcmp(resultType, APP_CONTROL_DATA_TYPE_PHONE) == 0) {
			return LIST_RESULT_NUMBER;
		} else if (strcmp(resultType, APP_CONTROL_DATA_TYPE_EMAIL) == 0) {
			return LIST_RESULT_EMAIL;
		} else if (strcmp(resultType, APP_CONTROL_DATA_TYPE_VCARD) == 0) {
			return LIST_RESULT_VCARD;
		} else if (strcmp(resultType, APP_CONTROL_DATA_TYPE_PHONE_OR_EMAIL) == 0) {
			return LIST_RESULT_NUMBER_OR_EMAIL;
		}

		free(resultType);
	}

	return LIST_RESULT_ID;
}

int CtListApp::getSelectLimit(app_control_h request)
{
	int limit = 0;
	char *limitStr = NULL;

	app_control_get_extra_data(request, APP_CONTROL_DATA_LIMIT, &limitStr);
	if (limitStr) {
		limit = atoi(limitStr);
		free(limitStr);
	}

	return limit;
}

void CtListApp::replyFailure()
{
	app_control_h reply;
	app_control_create(&reply);
	app_control_reply_to_launch_request(reply, m_Request, APP_CONTROL_RESULT_FAILED);
	app_control_destroy(reply);
}

void CtListApp::replySingleResult(const char *data, CtListResultType type)
{
	WHIT();
	app_control_h reply = NULL;
	app_control_create(&reply);

	if (data) {
		app_control_add_extra_data(reply, APP_CONTROL_DATA_TYPE, resultTypes[type]);
		app_control_add_extra_data_array(reply, APP_CONTROL_DATA_SELECTED, &data, 1);
	}

	app_control_reply_to_launch_request(reply, m_Request, APP_CONTROL_RESULT_SUCCEEDED);
	app_control_destroy(reply);
}

void CtListApp::replyMultiResult(const char *data[], size_t count)
{
	app_control_h reply = NULL;
	app_control_create(&reply);

	if (data) {
		app_control_add_extra_data(reply, APP_CONTROL_DATA_TYPE, resultTypes[m_ResultType]);
		app_control_add_extra_data_array(reply, APP_CONTROL_DATA_SELECTED, data, count);
	}

	app_control_reply_to_launch_request(reply, m_Request, APP_CONTROL_RESULT_SUCCEEDED);
	app_control_destroy(reply);
}
