/*
 * Copyright (c) 2009-2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#include <contacts.h>
#include <notification.h>

#include "ContactsDebug.h"
#include "CtProgressBarPopup.h"
#include "CtImporter.h"
#include "CtString.h"
#include "CtType.h"

CtImporter::CtImporter(int abId, std::vector<std::string> vcards, int contactsCount)
: __addressBookId(abId)
, __canceled(false)
, __paused(false)
, __importThread(NULL)
, __vcards(std::move(vcards))
, __contactsCount(contactsCount)
{
	WHIT();
	__createProgressPopup();
	__createCancelPopup();
}

void CtImporter::__createProgressPopup()
{
	__progressPopup = new CtProgressBarPopup(__contactsCount, V_("IDS_PB_HEADER_IMPORT_CONTACTS_ABB2"), true);
	auto stopFunction = std::bind(&CtImporter::__stopImporting, this, std::placeholders::_1);
	__progressPopup->addButton(V_("IDS_PB_BUTTON_CANCEL"), stopFunction);
	__progressPopup->setOnBack(stopFunction);
	__popupMonitor = __progressPopup->getWeakPtr();
}

void CtImporter::__createCancelPopup()
{
	__cancelPopup = new WPopup();
	__cancelPopup->setTextTranslatable(PACKAGE);
	__cancelPopup->setTitle("IDS_PB_HEADER_CANCEL_IMPORTING_ABB");

	auto continueFunction = std::bind(&CtImporter::__continueImporting, this, std::placeholders::_1);
	__cancelPopup->addButton("IDS_PB_BUTTON_CANCEL", continueFunction);
	__cancelPopup->setOnBack(continueFunction);

	auto cancelFunction = std::bind(&CtImporter::__cancelImporting, this, std::placeholders::_1);
	__cancelPopup->addButton("IDS_PB_BUTTON_OK_ABB2", cancelFunction);
}

CtImporter::~CtImporter()
{
	WHIT();
	if (auto popup = __popupMonitor.lock()) {
		popup->destroy();
	}
	if (__cancelPopup) {
		__cancelPopup->destroy();
	}
}

CtProgressBarPopup *CtImporter::getProgressPopup()
{
	return __progressPopup;
}

WPopup *CtImporter::getCancelPopup()
{
	return __cancelPopup;
}

void CtImporter::run()
{
	WHIT();
	__canceled = false;
	__paused = false;
	__importThread = ecore_thread_run(__importFromVcardsCb, __importFromVcardsEndCb, __importFromVcardsCancelCb, this);
}

void CtImporter::__stopImporting(bool* destroyPopup)
{
	WHIT();
	__paused = true;
	*destroyPopup = false;
	char content[CT_TEXT_MAX_LEN] = {0};
	char percent[CT_TEXT_SHORT_LEN] = {0};
	snprintf(percent, CT_TEXT_SHORT_LEN, "%d%%", (int)(100.0 * (double)(__importedContacts.size())/(double(__contactsCount))));
	snprintf(content, CT_TEXT_MAX_LEN, V_("IDS_PB_POP_CONTACTS_WILL_STOP_BEING_IMPORTED_FROM_PS"), percent);
	elm_object_text_set(__cancelPopup->getEvasObj(), content);
	evas_object_hide(__progressPopup->getEvasObj());
	evas_object_show(__cancelPopup->getEvasObj());
}

void CtImporter::__cancelImporting(bool* destroyPopup)
{
	WHIT();
	__paused = false;
	__canceled = true;
	*destroyPopup = true;
	__cancelPopup = nullptr;
}

void CtImporter::__continueImporting(bool* destroyPopup)
{
	WHIT();
	__paused = false;
	*destroyPopup = false;
	evas_object_show(__progressPopup->getEvasObj());
	evas_object_hide(__cancelPopup->getEvasObj());
}

void CtImporter::__importFromVcardsCb(void *data, Ecore_Thread *thread)
{
	WHIT();
	CtImporter* importer = static_cast<CtImporter*>(data);

	for (auto &&vcard : importer->__vcards) {
		if (importer->__canceled) {
			WDEBUG("Import canceled");
			ecore_thread_cancel(importer->__importThread);
			return;
		}
		if (importer->__paused) {
			WDEBUG("Import paused");
			while (importer->__paused) {
				usleep(1000);
			}
		}

		int err = contacts_vcard_parse_to_contact_foreach(vcard.c_str(), __vcardParseCb, importer);
		if (CONTACTS_ERROR_NONE != err) {
			WERROR("contacts_vcard_parse_to_contact_foreach failed: %d", err);
		}
	}
}

void CtImporter::__importFromVcardsEndCb(void* data, Ecore_Thread* thread)
{
	WHIT();
	CtImporter* importer = static_cast<CtImporter*>(data);
	int count = importer->__importedContacts.size();

	if (count > 0) {
		char buf[256] = { 0, };
		if (count == 1) {
			snprintf(buf, sizeof(buf), V_("IDS_PB_TPOP_1_CONTACT_IMPORTED"));
		} else {
			snprintf(buf, sizeof(buf), ctGetContactsString("IDS_PB_TPOP_PD_CONTACTS_IMPORTED"), count);
		}
		WDEBUG("notification: %s", buf);
		int err = notification_status_message_post(buf);
		if (err != NOTIFICATION_ERROR_NONE) {
			WERROR("notification_status_message_post: %d", err);
		}
	}

	delete importer;
}

void CtImporter::__importFromVcardsCancelCb(void* data, Ecore_Thread* thread)
{
	WHIT();
	CtImporter* importer = static_cast<CtImporter*>(data);

	for (auto &&id : importer->__importedContacts) {
		contacts_db_delete_record(_contacts_contact._uri, id);
	}

	int err = notification_status_message_post( V_("IDS_PB_SBODY_IMPORTING_CANCELLED_M_STATUS_ABB") );
	if (err != NOTIFICATION_ERROR_NONE) {
		WERROR("notification_status_message_post failed: %d", err);
	}

	delete importer;
}

bool CtImporter::__vcardParseCb(contacts_record_h record, void* user_data)
{
	WHIT();
	CtImporter *importer = static_cast<CtImporter *>(user_data);

	if (record) {
		if (importer->__canceled) {
			ecore_thread_cancel(importer->__importThread);
			return false;
		}

		if (importer->__paused) {
			WDEBUG("Import paused");
			while (importer->__paused) {
				usleep(1000);
			}
		}

		int id = 0;

		int err = contacts_record_set_int(record, _contacts_contact.address_book_id, importer->__addressBookId);
		if (CONTACTS_ERROR_NONE != err) {
			WERROR("contacts_record_set_int failed: %d", err);
		}

		err = contacts_db_insert_record(record, &id);
		if ((CONTACTS_ERROR_NONE != err) || (id <= 0)) {
			WERROR("contacts_db_insert_record failed: err = %d, id = %d", err, id);
		}

		importer->__importedContacts.push_back(id);
		importer->__progressPopup->setProgressStatusCount(importer->__importedContacts.size());
	}

	// Return true to continue to scan next contact, according to contacts_vcard_parse_cb specification.
	return true;
}
