/*
 * Copyright (c) 2009-2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#ifndef __CT_LIST_MODEL_DB_H_
#define __CT_LIST_MODEL_DB_H_

#include <map>
#include <vector>
#include <string>
#include <contacts.h>
#include <memory>
#include <Ecore.h>
#include <Elementary.h>
#include <mutex>

#include "ContactsCommon.h"
#include "ICtChangeListener.h"
#include "CtCommon.h"
#include "CtType.h"
#include "CtThread.h"

#include "CtListModelListener.h"
#include "CtListModel.h"
#include "CtListDataPerson.h"

#include "CtListModelFilter.h"


#define CTLISTDATA_NQUERY_COUNT 20
#define CTLISTDATA_NQUERY_TIMER_INTERVAL 0.4

class CtListModelDb : public CtListModel , public CtListModelListener
{
public:
	CtListModelDb(CtListHasType filterType, int limit);
	CtListModelDb(CtListDataType dataType);

	virtual ~CtListModelDb();

public:
	// CtListModel interfaces
	virtual int getContactsCount(CtListQueryType queryType);
	virtual int getContacts(CtListQueryType queryType, CtGetContactsType syncType);
	virtual int cancelGetContacts();
	virtual std::shared_ptr<CtListDataPerson> getAtPersonData(int index, CtListQueryType queryType);
	virtual int getPersonListCount(CtListQueryType queryType);
	virtual bool getMadePersonListStatus();
	virtual void attachListener(ICtChangeListener* listener);

	virtual void setSelectedPersonId(int selectedPersonId);
	virtual const char* getSearchString(void);
	virtual bool setSearchString(char* search);
	virtual void pushCheckedData(std::shared_ptr<CtListDataPerson> dataPerson);
	virtual void removeCheckedData(std::shared_ptr<CtListDataPerson> dataPerson);
	virtual void getNumberSearchString(const std::shared_ptr<CtListDataPerson>& dataPerson, std::string &numberStr);
	virtual int getCheckedDataCount();
	virtual int getLimitCount();
	virtual int getCurrentLinkedCount();

	// CtListModelListener
	virtual void onFilterChanged();

	std::vector<std::shared_ptr<CtListDataPerson>>* getCheckedDataList() const;
	std::map<int, CtAddressbookInfo*>& getAddressbookList();
	void setDisplayAddressbook(int addressbookId, bool display);
	bool checkSimAddressbook(int addressbookId);

	virtual void initialOriginalCheckedIdList();
	virtual void clearOriginalCheckedIdList();
	virtual bool checkIsSelectAllOnSearching();
	virtual void initialOriginalPersonDataListSize();
	virtual void clearOriginalPersonDataListSize();
	virtual int getOriginalPersonDataListSize();

	bool checkMostFrequentContact(int personId);

	void setSelectAllState(bool selectAll);
	bool getSelectAllState();
	void doCheckedDataNoRelation(std::shared_ptr<CtListDataPerson> dataPerson, bool check);



private:

	void __initialize(void);
	int __getQueryResult(int offset, int limit);
	CtListDataPerson* __createPersonData(contacts_record_h record);
	bool __makePersonList(contacts_list_h list, int offset);
	void __getMfcPersonListFromPreference();
	int __setImage(contacts_record_h record, char *image);
	bool __setMFCImage(contacts_record_h personRecord, char *imagePath);
	void __getMfcPersonList();
	void __getFavoritePersonList();
	bool __makeMfcPersonList(contacts_list_h list);
	bool __makeFavoritePersonList(contacts_list_h list);
	void __setAsCheck(std::shared_ptr<CtListDataPerson> personData, bool check, bool noRelation);
	bool __checkToDisplayMfc(char *addressbookIds);

private:
	static void __onDbChanged(const char* view_uri, void* user_data);
	static Eina_Bool __onGetContactsIdlerCb(void* data);
	static void __onGetContactsThreadRunCb(void *data);
	static void __onGetContactsEndCb(void *data);
	static void __onDisplayNameOrderChangedCb(contacts_name_display_order_e name_display_order, void* user_data);
	static void __onNameSortingOrderChangedCb(contacts_name_sorting_order_e name_display_order, void* user_data);
	static Eina_Bool __getCurentMfcListIdlerCb(void* data);
	static void __onGetCurrentMfcListThreadRunCb(void* data);
	static void __onGetCurrentMfcListThreadEndCb(void* data);

private:
	std::string __search;
	CtListDataType __dataType;
	ICtChangeListener *__listener;

	CtThread *__listThread;
	CtThread *__mfcThread;

	Ecore_Idler *__idler;
	Ecore_Idler *__updateMFCListIdler;
	int __offset;
	CtListModelFilter *__dataFilter;
	int __selectedPersonId;
	int __limitCount;
	int __currentLinkedCount;
	bool __madePersonList;
	bool __isSelectAll;
	int __excludeAddressbookId;
	bool __needUpdate;
	int __originalCount;
	int __curCheckCount;

	std::vector<std::pair<int, std::shared_ptr<CtListDataPerson>>> __personDataList;
	std::vector<std::pair<int, std::shared_ptr<CtListDataPerson>>> __mfcPersonDataList;
	std::vector<std::pair<int, std::shared_ptr<CtListDataPerson>>> __favoritePersonDataList;

	std::map<int, std::shared_ptr<CtListDataPerson>> __checkedDataList;
	std::map<int, std::shared_ptr<CtListDataPerson>> __checkedDataListOld;

	std::vector<int> __excludeIdList;
	std::vector<int> __originalCheckedIdList; // when searching, use this list to check
	int __originalPersonDataListSize; // when searching, use this size to check

	std::vector<CtAddressbookInfo*>	__addressbookList;

	std::mutex __mutexLock;
};

#endif

