/*
 * Copyright (c) 2009-2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#include <algorithm>
#include <contacts.h>
#include <account.h>
#include <app_preference.h>
#include "ContactsDebug.h"
#include "CtSettingData.h"
#include "CtSettingDataListener.h"
#include "CtType.h"
#include "CtString.h"

CtSettingData::AbInfo::AbInfo()
: abId(0)
, accountId(0)
, display(false)
{

}

CtSettingData::SdnInfo::SdnInfo()
: simNum(0)
{

}

CtSettingData::CtSettingData()
: __sortMode(SORT_MODE_FIRST_NAME)
, __nameOrder(NAME_ORDER_LAST_NAME_FIRST)
, __showMode(SHOW_MODE_ALL_CONTACTS)
, __sendMode(SEND_MODE_TOGETHER)
, __displayAll(false)
{
	WHIT();
}

CtSettingData::~CtSettingData()
{
	WHIT();
}

void CtSettingData::init()
{
	// Load data from preference

	int showMode = 0;
	preference_get_int(CT_KEY_DISPLAY_WITH_NUMBER, &showMode);
	if (showMode == 0)
		__showMode = SHOW_MODE_ALL_CONTACTS;
	else
		__showMode = SHOW_MODE_CONTACTS_WITH_PHONES;

	contacts_name_sorting_order_e sortOrder = CONTACTS_NAME_SORTING_ORDER_FIRSTLAST;
	contacts_setting_get_name_sorting_order(&sortOrder);
	if (sortOrder == CONTACTS_NAME_SORTING_ORDER_FIRSTLAST)
		__sortMode = SORT_MODE_FIRST_NAME;
	else
		__sortMode = SORT_MODE_LAST_NAME;

	contacts_name_display_order_e displayOrder = CONTACTS_NAME_DISPLAY_ORDER_FIRSTLAST;
	contacts_setting_get_name_display_order(&displayOrder);
	if (displayOrder == CONTACTS_NAME_DISPLAY_ORDER_FIRSTLAST)
		__nameOrder = NAME_ORDER_FIRST_NAME_FIRST;
	else
		__nameOrder = NAME_ORDER_LAST_NAME_FIRST;

	int sendMode = 0;
	preference_get_int(CT_KEY_SETTING_CONTACT_SHARING_SETTINGS, &sendMode);
	if (sendMode == 0)
		__sendMode = SEND_MODE_TOGETHER;
	else
		__sendMode = SEND_MODE_INDIVIDUALLY;

	__initAddressbookInfo();
}

void CtSettingData::addListener(CtSettingDataListener* listener)
{
    __listeners.push_back(listener);
}

void CtSettingData::__notify()
{
    for (auto iter = __listeners.begin(); iter != __listeners.end(); iter++)
    {
		if(*iter) {
			(*iter)->onChanged();
		}
    }
}
void CtSettingData::__initAddressbookInfo()
{
	contacts_list_h abList = NULL;
	contacts_record_h addressbook = NULL;
	char* abName = NULL;
	int abId = 0;
	int accountId = 0;
	std::vector<int> displayAbId;
	bool displayAll = false;
	int mode = 0;
	char *str = NULL;

	preference_get_string(CT_KEY_LIST_DISPLAY_ADDRESSBOOK, &str);

	WDEBUG("display addressbook %s", str);

	if (SAFE_STRLEN(str) == 0)
	{
		__displayAll = true;
		displayAll = true;
	}
	else
	{
		WDEBUG("ab: %s", str);
		char* ptr = NULL;
		char* token = strtok_r(str, ";", &ptr);
		while (token)
		{
			displayAbId.push_back(atoi(token));

			token = strtok_r(NULL, ";", &ptr);
		}


	}
	free(str);
	str = NULL;

	int ret = contacts_db_get_all_records(_contacts_address_book._uri, 0, 0, &abList);
	if(ret != CONTACTS_ERROR_NONE)
	{
		WWARNING("contacts_db_get_all_records failed");
		if(abList)
			contacts_list_destroy(abList, true);
		return;
	}
	int count = 0;
	contacts_list_get_count(abList, &count);
	if (count <= 0){
		if(abList)
			contacts_list_destroy(abList, true);
		return;
	}
	for (int i = 0; i < count; i++)
	{
		ret = contacts_list_get_current_record_p(abList, &addressbook);
		WPWARN(CONTACTS_ERROR_NONE != ret, "contacts_list_get_current_record_p() Failed(%d)", ret);

		ret = contacts_record_get_int(addressbook, _contacts_address_book.id, &abId);
		WPWARN(CONTACTS_ERROR_NONE != ret, "contacts_record_get_int() Failed(%d)", ret);
		ret = contacts_record_get_int(addressbook, _contacts_address_book.account_id, &accountId);
		WPWARN(CONTACTS_ERROR_NONE != ret, "contacts_record_get_int() Failed(%d)", ret);
		ret = contacts_record_get_int(addressbook, _contacts_address_book.mode, &mode);
		WPWARN(CONTACTS_ERROR_NONE != ret, "contacts_record_get_int() Failed(%d)", ret);
		ret = contacts_record_get_str_p(addressbook, _contacts_address_book.name, &abName);
		WPWARN(CONTACTS_ERROR_NONE != ret, "contacts_record_get_str_p() Failed(%d)", ret);

		WDEBUG("adId %d, name %s", abId, abName);

		AbInfo info;
		info.abId = abId;
		info.accountId = accountId;
		info.mode = mode;
		if (abName)
			info.abName = abName;

		if (displayAll)
			info.display = true;
		else if (std::find(displayAbId.begin(), displayAbId.end(), info.abId) != displayAbId.end())
		{
			info.display = true;
		}
		else
		{
			info.display = false;
		}

		if (accountId > 0)
		{
			account_h account = NULL;
			account_create(&account);
			char* str = NULL;

			account_query_account_by_account_id(accountId, &account);
			if (account)
			{
				char* appId = NULL;
				account_get_user_name(account, &str);
				if (str)
				{
					info.user = str;
					free(str);
					str = NULL;
				}

				account_get_package_name(account, &appId);
				account_type_query_label_by_app_id([](char* app_id, char* label, char* locale, void* user_data)->bool{
					AbInfo* pInfo = static_cast<AbInfo*>(user_data);
					if(strcmp(locale,"default") == 0){
						pInfo->name = label;
						return false;
					}

					return true;
				}, appId, &info);

				if (info.name.empty())
				{
					account_get_domain_name(account, &str);
					if (str)
					{
						info.name = str;
						free(str);
					}
				}

				account_destroy(account);
			}

			if(!strcmp(info.name.c_str(), CT_EAS_ADDRESSBOOK_NAME)) {
				info.name.clear();
				info.name = CT_EAS_ADDRESSBOOK_DISPLAY_NAME;
				info.user.clear();
				info.user = abName;
			}

			__abInfo.push_back(info);
		}

		contacts_list_next(abList);

		WDEBUG("name:%s, user:%s", info.name.c_str(), info.user.c_str());
	}

	contacts_list_destroy(abList, true);
}

CtSettingData::SortMode CtSettingData::getSortMode() const
{
	return __sortMode;
}

void CtSettingData::setSortMode(SortMode sortMode)
{
	if (sortMode == SORT_MODE_FIRST_NAME)
		contacts_setting_set_name_sorting_order(CONTACTS_NAME_SORTING_ORDER_FIRSTLAST);
	else
		contacts_setting_set_name_sorting_order(CONTACTS_NAME_SORTING_ORDER_LASTFIRST);

	__sortMode = sortMode;
}

CtSettingData::NameOrder CtSettingData::getNameOrder() const
{
	return __nameOrder;
}

void CtSettingData::setNameOrder(NameOrder nameOrder)
{
	if (nameOrder == NAME_ORDER_FIRST_NAME_FIRST)
		contacts_setting_set_name_display_order(CONTACTS_NAME_DISPLAY_ORDER_FIRSTLAST);
	else
		contacts_setting_set_name_display_order(CONTACTS_NAME_DISPLAY_ORDER_LASTFIRST);

	__nameOrder = nameOrder;
}

CtSettingData::ShowMode CtSettingData::getShowMode() const
{
	return __showMode;
}

void CtSettingData::setShowMode(ShowMode showMode)
{
	if (showMode == SHOW_MODE_CONTACTS_WITH_PHONES)
		preference_set_int(CT_KEY_DISPLAY_WITH_NUMBER, 1);
	else
		preference_set_int(CT_KEY_DISPLAY_WITH_NUMBER, 0);

	__showMode = showMode;
}

CtSettingData::SendMode CtSettingData::getSendMode() const
{
	return __sendMode;
}

void CtSettingData::setSendMode(SendMode mode)
{
	__sendMode = mode;

	if (__sendMode == SEND_MODE_TOGETHER) {
		preference_set_int(CT_KEY_SETTING_CONTACT_SHARING_SETTINGS, 0);
	} else {
		preference_set_int(CT_KEY_SETTING_CONTACT_SHARING_SETTINGS, 1);
	}

    __notify();
}

const std::list<CtSettingData::AbInfo>& CtSettingData::getAbInfo() const
{
	return __abInfo;
}

void CtSettingData::setAbInfo(const std::list<CtSettingData::AbInfo>& abInfo)
{
	WHIT();
	__abInfo = abInfo;

	char buf[256]={0,};
	int len = 0;
	bool displayAll = true;

	for (auto iter = __abInfo.begin(); iter != __abInfo.end(); ++iter)
	{
		if (iter->display)
		{
			if (len > 0)
				len += snprintf(buf + len, sizeof(buf) - len, ";%d", iter->abId);
			else
				len += snprintf(buf + len, sizeof(buf) - len, "%d", iter->abId);
		}
		else
		{
			displayAll = false;
		}
	}

	WDEBUG("abInfo %s", buf);

	if (displayAll)
	{
		__displayAll = true;
		preference_set_string(CT_KEY_LIST_DISPLAY_ADDRESSBOOK, "");
	}
	else
	{
		__displayAll = false;
		preference_set_string(CT_KEY_LIST_DISPLAY_ADDRESSBOOK, buf);
	}
}

int CtSettingData::getAbCount() const
{
	return __abInfo.size();
}

bool CtSettingData::isDisplayAll() const
{
	return __displayAll;
}

const std::list<CtSettingData::SdnInfo>& CtSettingData::getSdnInfoList()
{
	WHIT();
	if (__sdnInfo.size() == 0)
	{
		contacts_list_h sdnList = NULL;
		contacts_record_h sdnRecord = NULL;
		char* strValue = NULL;
		int intValue = 0;
		int err = CONTACTS_ERROR_NONE;

		err = contacts_db_get_all_records(_contacts_sdn._uri, 0, 0, &sdnList);
		WPWARN(CONTACTS_ERROR_NONE != err, "contacts_db_get_all_records() Failed(%d)", err);

		while (CONTACTS_ERROR_NONE == contacts_list_get_current_record_p(sdnList, &sdnRecord))
		{
			SdnInfo sdnInfo;

			err = contacts_record_get_str_p(sdnRecord, _contacts_sdn.name, &strValue);
			WPWARN(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() Failed(%d)", err);
			if (strValue)
				sdnInfo.name = strValue;

			err = contacts_record_get_str_p(sdnRecord, _contacts_sdn.number, &strValue);
			WPWARN(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() Failed(%d)", err);
			if (strValue)
				sdnInfo.number = strValue;

			err = contacts_record_get_int(sdnRecord, _contacts_sdn.sim_slot_no, &intValue);
			WPWARN(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);
			sdnInfo.simNum = intValue;

			__sdnInfo.push_back(sdnInfo);

			contacts_list_next(sdnList);
		}

		contacts_list_destroy(sdnList, true);
	}

	WDEBUG("count:%d", __sdnInfo.size());

	return __sdnInfo;
}
