/*
 * Copyright (c) 2009-2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */


#include <memory>
#include <vector>
#include <app_preference.h>
#include <contacts.h>
#include <bundle.h>
#include <system_settings.h>

#include "ContactsApp.h"

#include "WAppWindow.h"
#include "WNaviframe.h"

#include "ContactsAppData.h"
#include "ContactsDebug.h"
#include "ContactsCommon.h"

#include "PhLogMainListView.h"
#include "CtListView.h"
#include "CtTabView.h"
#include "badge.h"
#include "ContactsAppControlLauncher.h"

#define CT_AUL_INTERNAL_APPID "__AUL_INTERNAL_APPID__"
#define V_(str) dgettext("contacts", str)

ContactsApp::ContactsApp()
{
	WHIT();
	__tabView = NULL;
	__contactsData = NULL;

	__selectedView = TAB_VIEW_NONE;
	__logsLaunchingTimer = NULL;
	__dialerView = NULL;
	contacts_connect();
}

ContactsApp::~ContactsApp()
{
	WHIT();
	if(__logsLaunchingTimer) {
		ecore_timer_del(__logsLaunchingTimer);
		__logsLaunchingTimer = NULL;
	}
	contacts_disconnect();

}

void ContactsApp::onPause()
{
	WHIT();
}

void ContactsApp::onResume()
{
	WHIT();
	ContactsAppControlLauncher::getInstance().unblockUi();
}

bool ContactsApp::onCreate()
{
	WENTER();

	char *locale = NULL;
	system_settings_get_value_string(SYSTEM_SETTINGS_KEY_LOCALE_LANGUAGE, &locale);
	if (locale) {
		elm_language_set(locale);
		free(locale);
	}

	__languageChangedHandler.addEventHandler( APP_EVENT_LANGUAGE_CHANGED, [](app_event_info_h eventInfo, void* userData ) {
			WHIT();
			char* lang = NULL;
			app_event_get_language( eventInfo, &lang );

			if( lang )
				elm_language_set( lang );
			free(lang);
	}, this );

	WWindow* p = new WAppWindow("Phone-Contacts", ELM_WIN_BASIC);
	attachWindow(p);

	elm_app_base_scale_set(2.6);

	getWindow()->attachBaseUiObject(new WNaviframe());

	WLEAVE();

	return true;
}

void ContactsApp::onTerminate()
{
	WHIT();

	if( __contactsData ) {
		free(__contactsData->calleeId);
		free( __contactsData );
	}
	__contactsData = NULL;
}

void ContactsApp::__pushJob(void* data)
{
	WDEBUG("push");
	ContactsApp* app = (ContactsApp*)data;
	WNaviframe* frame = (WNaviframe*)app->getWindow()->getBaseUiObject();

	std::shared_ptr<std::vector<CtTabSubView>> subView(new std::vector<CtTabSubView>);
	subView->push_back(TAB_VIEW_DIALER);
	subView->push_back(TAB_VIEW_LOGS);
	subView->push_back(TAB_VIEW_CONTACTS);

	app->__tabView = new CtTabView(app->__selectedView, false, app->__contactsData);

	std::string number = app->__getAppControlPhoneNumber(app->__contactsData->appControl);
	IContactsTabPageView* view = new PhDialerView;
	view->setName("Dialer");
	app->__tabView->insert(TAB_VIEW_DIALER, view);
	app->__dialerView = (PhDialerView*)view;
	app->__dialerView->setNumber(number);
	view = new PhLogMainListView(app->__contactsData);
	view->setName("Logs");
	app->__tabView->insert(TAB_VIEW_LOGS, view);

	view = new CtListView(LIST_VIEW_DEFAULT, LIST_DATA_DEFAULT, LIST_CTRL_DEFAULT);
	view->setName("Contacts");
	app->__tabView->insert(TAB_VIEW_CONTACTS, view);

	frame->push(app->__tabView);
	WDEBUG("pushed");
	return;
}

void ContactsApp::__firstResetJob(void* data)
{
	WDEBUG("reset on first launch");
	ContactsApp* app = (ContactsApp*)data;
	app->__tabView->reset(app->__selectedView);
	app->__selectedView = TAB_VIEW_NONE;
	if (app->__contactsData->appControl) {
		app_control_destroy(app->__contactsData->appControl);
		app->__contactsData->appControl = NULL;
	}
	WDEBUG("reseted");
	return;
}

void ContactsApp::__secondResetJob(void* data)
{
	WDEBUG("reset on second launch");
	ContactsApp* app = (ContactsApp*)data;
	app->__tabView->reset(app->__selectedView);

	if (app->__contactsData->appControl) {
		char *operation = NULL;
		app_control_get_operation(app->__contactsData->appControl, &operation);
		if (operation && (strcmp(operation, APP_CONTROL_OPERATION_DIAL) == 0)) {
			std::string number = app->__getAppControlPhoneNumber(app->__contactsData->appControl);
			app->__dialerView->setNumber(number);
		}
		free(operation);

		app_control_destroy(app->__contactsData->appControl);
		app->__contactsData->appControl = NULL;
	}
	WDEBUG("reseted");
	return;
}

void ContactsApp::onAppControl(app_control_h request, bool firstLaunch)
{
	WENTER();

	WDEBUG("app_control_h %p", request);
	CtTabSubView selectedView = TAB_VIEW_DIALER;

	if(firstLaunch) {
		WINFO("it is first launch!");
		WASSERT(__tabView == NULL);

		__contactsData = (ContactsData*)calloc(1, sizeof(ContactsData));
		WPRET_M(!__contactsData, "calloc() failed");
		app_control_clone(&__contactsData->appControl, request);
		app_control_get_app_id(__contactsData->appControl, &__contactsData->calleeId);
		WINFO("calleeId %s", __contactsData->calleeId);

		ContactsCommon::loadLocalization();
		if(__contactsData->calleeId && strcmp(__contactsData->calleeId, CONTACTS_PKG) == 0) {
			selectedView = TAB_VIEW_CONTACTS;
		}
		else if((__contactsData->calleeId && strcmp(__contactsData->calleeId, PHONE_PKG) == 0 ) ||
				ContactsCommon::getAppControlData(request, CONTACTS_APPCONTROL_KEY_OPERATION).compare(APP_CONTROL_OPERATION_DIAL) == 0) {
			selectedView = __decideTabforPhone(__contactsData->appControl);

			if (selectedView == TAB_VIEW_LOGS
				|| selectedView == TAB_VIEW_LOGS_CHECK) {
				ecore_thread_run([](void *data, Ecore_Thread *thread) {
					//ContactsCommon::addNumberDashes("1");
				}, NULL, NULL, NULL);
			}
		} else if(!__contactsData->calleeId) {
			selectedView = TAB_VIEW_CONTACTS;
		} else {
			WERROR("wrong package %s", __contactsData->calleeId);
		}

		__selectedView = selectedView;

		ecore_job_add(__pushJob, this);
		ecore_job_add(__firstResetJob, this);
	}
	else {
		WDEBUG("it is not first launch!");
		free(__contactsData->calleeId);
		app_control_clone(&__contactsData->appControl, request);
		app_control_get_app_id(__contactsData->appControl, &__contactsData->calleeId);
		WINFO("calleeId %s", __contactsData->calleeId);
		if((__contactsData->calleeId && 0 == strcmp(__contactsData->calleeId, PHONE_PKG)) ||
			ContactsCommon::getAppControlData(request, CONTACTS_APPCONTROL_KEY_OPERATION).compare(APP_CONTROL_OPERATION_DIAL) == 0) {
			selectedView = __decideTabforPhone(__contactsData->appControl);
		}
		else if(__contactsData->calleeId && 0 == strcmp(__contactsData->calleeId, CONTACTS_PKG)) {
			selectedView = TAB_VIEW_CONTACTS;
		}
		else {
			WERROR("no callee id");
		}

		__selectedView = selectedView;

		ecore_job_add(__secondResetJob, this);
	}

	//important, this line should be exist actually. but in case contact app, it does not need.
	//becasue, in some part, already handled by other operations
	//especially this is for launch performance
	//WApp::onAppControl(request, firstLaunch);
	WLEAVE();
}

CtTabSubView ContactsApp::__decideTabforPhone(app_control_h appControl)
{
	CtTabSubView app = TAB_VIEW_NONE;

	if (ContactsCommon::getAppControlData(appControl, CONTACTS_APPCONTROL_KEY_OPERATION).compare(APP_CONTROL_OPERATION_DIAL) == 0)
		app = TAB_VIEW_DIALER;
	else if (ContactsCommon::getAppControlData(appControl, "MMICHECK_SEND").compare("1") == 0)
		app = TAB_VIEW_DIALER;
	else if (__isMissedCallExist())
	{
		 if (ContactsCommon::getAppControlData(appControl, "CALLER").compare("lock_screen") == 0)
		{
				__logsLaunchingTimer = ecore_timer_add(0.3, [](void* data)->Eina_Bool {
					ContactsApp *p = (ContactsApp *)data;
					app_control_h reply;
					app_control_create(&reply);
					app_control_add_extra_data(reply, "destroy_request", "true");
					app_control_reply_to_launch_request(reply, p->__contactsData->appControl, APP_CONTROL_RESULT_SUCCEEDED);
					app_control_destroy(reply);
					p->__logsLaunchingTimer = NULL;
		 		return ECORE_CALLBACK_CANCEL;
		}, this);
		 }
		app = TAB_VIEW_LOGS;
	}
	else if (ContactsCommon::getAppControlData(appControl, "launch_type").compare("log") == 0)
		app = TAB_VIEW_LOGS;
	else if (ContactsCommon::getAppControlData(appControl, "logs").compare("missed_call") == 0)
		app = TAB_VIEW_LOGS;
	else {//(app == TAB_VIEW_NONE)
		int error = preference_get_int(CT_KEY_LAST_SELECTED_TAB_INDEX_FOR_PHONE, (int*)&app);
		if (error != PREFERENCE_ERROR_NONE)
			app = TAB_VIEW_DIALER;
	}

	preference_set_int(CT_KEY_LAST_SELECTED_TAB_INDEX_FOR_PHONE, (int)app);
	WDEBUG("last selectedView saved as %d", app);

	return app;
}

bool ContactsApp::__isMissedCallExist()
{
	unsigned int count = 0;
	badge_get_count(PHONE_PKG, &count);
	return count > 0;
}

std::string ContactsApp::__getAppControlPhoneNumber(app_control_h appControl)
{
	std::string number;
	do {
		std::string number_with_prefix = ContactsCommon::getAppControlData(appControl, CONTACTS_APPCONTROL_KEY_URI);
		if(number_with_prefix.empty()) {
			break;
		}

		const char *prefix = "tel:";
		size_t pos = number_with_prefix.find(prefix);
		if(0 != pos) {
			WERROR("phone number format is invalid (%s)", number_with_prefix.c_str());
			break;
		}
		number = number_with_prefix.replace(number_with_prefix.begin(), number_with_prefix.begin() + strlen(prefix), "");

	} while (false);

	return number;
}
